(function($) {
    //height of the arrow image, in pixels
    var ARROW_IMG_HEIGHT = 8;

    //stores a global shadow reference that can be reused. Saves from making multiple $() calls when trying to obtain
    //references to the shadow div, and saves a creation of a template div in the body just to copy off.
    var globalShadow = $('<div id="inline-dialog-shadow"><div class="tl"></div><div class="tr"></div><div class="l"></div><div class="r"></div><div class="bl"></div><div class="br"></div><div class="b"></div></div>')

    //maps each trigger to its state, so that multiple triggers wont interfer with each other
    var getState = function($trigger) {
        var s = $trigger.data("__InlineDialogState") || {
            beingShown : false,
            shouldShow : false,
            contentLoaded : false,
            contentLoading : false,
            isOverPopup: false,
            targetPosition: null
        };
        $trigger.data("__InlineDialogState", s);
        return s;
    };
    //remember which element currently triggered the popup - use it to store information about the popup's state
    var getCurrentTrigger = function($popup) {
        return $popup.data("currentTrigger");
    };
    var updateCurrentTrigger = function($popup, $trigger) {
        $popup.data("currentTrigger", $trigger);
    };

    /**
     * Creates a new inline dialog.
     * The dialog is loaded lazily - that is, only created and attached to the dom when the event that
     * triggers it happens (e.g., mouse over one of the items that is passed in as a parameter).
     *
     * @param items jQuery object - the items that trigger the display of this popup when the user mouses over.
     * @param identifier A unique identifier for this popup. This should be unique across all popups on the page and a valid CSS class.
     * @param url The URL to retrieve popup contents.
     * @param options Custom options to change default behaviour. See AJS.InlineDialog.opts for default values and valid options.
     *
     * @return jQuery object - the popup that was created
     */
    AJS.InlineDialog = function(items, identifier, url, options) {
        var opts = $.extend(false, AJS.InlineDialog.opts, options);
        var $popup = $('<div id="inline-dialog-' + identifier + '" class="aui-inline-dialog"><div class="contents"></div></div>');
        var $arrow = $('<div id="arrow-' + identifier + '" class="arrow"></div>');
        if (opts.showArrow) {
            $arrow.appendTo($popup);
        }
        var $contents = $popup.find(".contents");
        $contents.css("width", opts.width + "px");
        var $thisPopupShadow = globalShadow.clone();

        var showPopup = function($trigger) {
            var state = getState($trigger);
            if (state.contentLoaded && state.shouldShow) {
                $(items).addClass("inline_dialog_active");
                state.beingShown = true;
                // retrieve the position of the click target. The offsets might be different for different types of targets and therefore
                // either have to be customisable or we will have to be smarter about calculating the padding and elements around it

                var posx = state.targetPosition.target.offset().left + opts.offsetX;
                var posy = state.targetPosition.target.offset().top + state.targetPosition.target.height() + opts.offsetY;
                posy += opts.showArrow ? ARROW_IMG_HEIGHT : 0;

                var diff = $(window).width() - (posx + opts.width + 30);
                if (diff < 0) {
                    $popup.css({
                        right: "20px",
                        left: "auto"
                    });
                    $arrow.css({
                        left: (-diff + (state.targetPosition.target.width() / 2)) + "px",
                        right: "auto"
                    });
                } else {
                    $popup.css({
                        left: posx + "px",
                        right: "auto"
                    });

                    $arrow.css({
                        left: state.targetPosition.target.width() / 2 + "px",
                        right: "auto"
                    });
                }

                var bottomOfViewablePage = (window.pageYOffset || document.documentElement.scrollTop) + $(window).height();
                if ((posy + $popup.height()) > bottomOfViewablePage) {
                    posy = bottomOfViewablePage - $popup.height() - 5;
                }
                $popup.css({
                    top: posy + "px"
                });

                var shadow = $thisPopupShadow.appendTo($popup).show();
                // reset position of popup box
                $popup.fadeIn(opts.fadeTime, function() {
                    // once the animation is complete, set the tracker variables
                    // beingShown = false; // is this necessary? Maybe only the shouldShow will have to be reset?
                });

                shadow.css({
                    width: $contents.outerWidth() + 32 + "px",
                    height: $contents.outerHeight() + 25 + "px"
                });
                $(".b", shadow).css("width", $contents.outerWidth() - 26 + "px");
                $(".l, .r", shadow).css("height", $contents.outerHeight() - 21 + "px");
            }
        };

        var doHidePopup = function (fadeoutCallback) {
            $(items).removeClass("inline_dialog_active");
            $popup.fadeOut(opts.fadeTime, function() {
                if (fadeoutCallback) {
                    fadeoutCallback();
                }
            });
        };

        var hidePopup = function($trigger, force) {
            updateCurrentTrigger($popup, $trigger);
            var state = getState($trigger);
            // only exectute the below if the popup is currently being shown
            if ((state.beingShown && !state.isOverPopup) || force) {
                state.hideDelayTimer = setTimeout(function() {
                    doHidePopup(function() {
                        opts.hideCallback.call({
                            popup: $popup,
                            hide: function () {
                                hidePopup($trigger);
                            },
                            id: identifier,
                            show: function () {
                                showPopup($trigger);
                            }
                        });
                    });
                    getState($trigger).shouldShow = false;
                }, opts.hideDelay);
            }
        };
        var clearPriorHideDelay = function() {
            var $t = getCurrentTrigger($popup);
            if ($t) {
                var state = getState($t);
                clearTimeout(state.hideDelayTimer);
            }
        };
        // the trigger is the jquery element that is triggering the popup (i.e., the element that the mousemove event is bound to)
        var initPopup = function(e, $trigger) {
            clearPriorHideDelay();
            updateCurrentTrigger($popup, $trigger);
            var state = getState($trigger);
            // Workaround for IE7: even though jQuery normalizes e.target, it seems we need to do it again here.
            state.targetPosition = { target: $(e.target || e.srcElement) };
            state.shouldShow = true;
            var doShowPopup = function() {
                var s = getState($trigger);
                var lastTrigger = getCurrentTrigger($popup);
                if (lastTrigger !== $trigger) {
                    //dont show the popup if the trigger is different to the one expected (i.e., mouse moved during the ajax load)
                    return;
                }
                if (!opts.cacheContent) {
                    s.contentLoading = false; //dont reset this flag so as to cache ajax content
                }
                s.contentLoaded = true;
                showPopup($trigger);
            };
            // lazy load popup contents
            if (!state.contentLoading) {
                state.contentLoading = true && opts.cacheContent;
                var doShowPopupWithInit = function() {
                    doShowPopup();
                    opts.initCallback.call({
                        popup: $popup,
                        hide: function () {
                            hidePopup($trigger);
                        },
                        id: identifier,
                        show: function () {
                            showPopup($trigger);
                        }
                    });
                };
                // If the passed in URL is a function, execute it. Otherwise simply load the content.
                if ($.isFunction(url)) {
                    url($contents, $trigger, doShowPopupWithInit);
                } else {
                    $contents.load(url, doShowPopupWithInit);
                }
            } else {
                doShowPopup();
            }
            return false;
        };
        var added = false;
        var appendPopup = function () {
            if (!added) {
                $(opts.container).append($popup);
                if (opts.onHover) {
                    var popupMouseoverHandler = function() {
                        var $trigger = getCurrentTrigger($popup);
                        if ($trigger) {
                            var state = getState($trigger);
                            state.isOverPopup = true;
                            clearTimeout(state.hideDelayTimer);
                        }
                    };
                    var popupMouseoutHandler = function() {
                        var $trigger = getCurrentTrigger($popup);
                        if ($trigger) {
                            getState($trigger).isOverPopup = false;
                            hidePopup($trigger);
                        }
                    };
                    $contents.bind("mouseenter.inlinedialogPopupHandler", popupMouseoverHandler)
                            .bind("mouseleave.inlinedialogPopupHandler", popupMouseoutHandler);
                }
                added = true;
            }
        };
        //
        // start of mouse event binding
        //
        if (opts.onHover) {
            var config = {
                sensitivity : opts.sensitivity, //max number of pixels moved between "interval" before registering as a hover
                interval: opts.showDelay,
                timeout: opts.showDelay,
                useLiveEvents: opts.useLiveEvents,
                over: function(e) {
                    appendPopup();
                    initPopup(e, $(this));
                },
                out: function(e) {
                    hidePopup($(this));
                }
            };
            $(items).hoverIntent(config);
        } else {
            var clickHandler = function(e) {
                var $trigger = $(this);
                if ($trigger.hasClass("inline_dialog_active")) {
                    hidePopup($trigger);
                } else {
                    appendPopup();
                    initPopup(e, $trigger);
                }
                return false;
            };
            if (opts.useLiveEvents) {
                $(items).live("click.inlinedialog", clickHandler);
            } else {
                $(items).click(clickHandler);
            }
        }

        // When merging to the latest version of AUI, don't revert to using e.stopPropagation()
        // on clicks inside the dialog (it prevents live events from working inside the dialog).
        $("body").bind("click.inlinedialog", function (e) {
            var $target = $(e.target);
            var clickedOutsideDialog = $target.closest('.aui-inline-dialog').length === 0;
            if (clickedOutsideDialog) {
                var isClickedOnIgnored = false;
                var ignoredElements = opts.ignoredElements;
                var length = ignoredElements.length;
                for (var i = 0; i < length; i++) {
                    if ($target.closest(ignoredElements[i]).length !== 0) {
                        isClickedOnIgnored = true;
                        break;
                    }
                }
                if (!isClickedOnIgnored) {
                    hidePopup($(this), true);
                    return;
                }
            }
        });

        return $popup;
    };

    AJS.InlineDialog.opts = {
        onHover: false,
        showArrow: true,
        fadeTime: 100,
        hideDelay: 10000,
        showDelay: 0,
        sensitivity: 2,
        width: 300,
        offsetX: 0,
        offsetY: 10,
        container: "body",
        cacheContent : true,
        useLiveEvents : false,
        ignoredElements : [], //a list of selector string which will match elements that should not cause this hover to hide when clicked on
        hideCallback: function() {
        }, // if defined, this method will be exected after the popup has been faded out.
        initCallback: function() {
        } // A function called after the popup contents are loaded. `this` will be the popup jQuery object, and the first argument is the popup identifier.
    };
})(jQuery);
