/*
 * Viewport - jQuery selectors for finding elements in viewport
 *
 * Copyright (c) 2008-2009 Mika Tuupola
 *
 * Licensed under the MIT license:
 *   http://www.opensource.org/licenses/mit-license.php
 *
 * Project home:
 *  http://www.appelsiini.net/projects/viewport
 *
 * Modifications:
 * - monitor the #frx-pane instead of the window
 * - threshold is an optional param, e.g., ':in-viewport(100)'
 * 
 * Optimizations:
 * - avoid redundant lookups
 * - avoid horizontal calculation (via ':in-viewport-vert')
 */
(function($) {

    var $container;
    var containerOffset;
    var containerScrollTop;
    var containerScrollLeft;
    var $element;
    var elementOffset;

    function init(element) {
        if (!$container) {
            // TODO Generalize this plugin to accept the container element as an argument
            $container = $('#frx-pane');
            containerOffset = $container.offset();
        }
        containerScrollTop = $container.scrollTop();
        containerScrollLeft = $container.scrollLeft();
        $element = $(element);

        var rawOffset = $element.offset();
        elementOffset = {
            top: rawOffset.top - containerOffset.top + containerScrollTop,
            left: rawOffset.left - containerOffset.left + containerScrollLeft
        };
    }

    function isBelowTheFold(settings) {
        var fold = $container.height() + containerScrollTop;
        return fold <= elementOffset.top - settings.threshold;
    }

    $.belowthefold = function(element, settings) {
        init(element);
        return isBelowTheFold(settings);
    };

    function isAboveTheTop(settings) {
        var top = containerScrollTop;
        return top >= elementOffset.top + $element.height() - settings.threshold;
    };

    $.abovethetop = function(element, settings) {
        init(element);
        return isAboveTheTop(settings);
    };

    function isRightOfScreen(settings) {
        var fold = $container.width() + containerScrollLeft;
        return fold <= elementOffset.left - settings.threshold;
    }

    $.rightofscreen = function(element, settings) {
        init(element);
        return isRightOfScreen(settings);
    };

    function isLeftOfScreen(settings) {
        var left = containerScrollLeft;
        return left >= elementOffset.left + $element.width() - settings.threshold;
    }

    $.leftofscreen = function(element, settings) {
        init(element);
        return isLeftOfScreen(settings);
    };

    $.inviewport = function(element, settings) {
        init(element);
        return !isLeftOfScreen(settings) && !isRightOfScreen(settings) && !isBelowTheFold(settings) && !isAboveTheTop(settings);
    };

    $.inviewportvert = function(element, settings) {
        init(element);
        return !isBelowTheFold(settings) && !isAboveTheTop(settings);
    };

    $.extend($.expr[':'], {
        "below-the-fold": function(a, i, m) {
            return $.belowthefold(a, {threshold : parseInt(m[3], 10) || 0});
        },
        "above-the-top": function(a, i, m) {
            return $.abovethetop(a, {threshold : parseInt(m[3], 10) || 0});
        },
        "left-of-screen": function(a, i, m) {
            return $.leftofscreen(a, {threshold : parseInt(m[3], 10) || 0});
        },
        "right-of-screen": function(a, i, m) {
            return $.rightofscreen(a, {threshold : parseInt(m[3], 10) || 0});
        },
        "in-viewport": function(a, i, m) {
            return $.inviewport(a, {threshold : parseInt(m[3], 10) || 0});
        },
        "in-viewport-vert": function(a, i, m) {
            return $.inviewportvert(a, {threshold : parseInt(m[3], 10) || 0});
        }
    });

})(jQuery);
